"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zhike_consul_1 = __importDefault(require("zhike-consul"));
const zignis_1 = require("zignis");
const consulConfig = require('../../../consul.json');
const env = zignis_1.Utils.getNodeEnv(); // development/production/test
const consulCachedKV = {};
const consulCachedInstance = {};
exports.command = 'consul <keys..>';
exports.desc = 'zhike consul config review';
exports.aliases = 'config';
exports.builder = function (yargs) {
    yargs.option('quiet', {
        alias: 'q',
        default: false,
        describe: 'simple console log'
    });
    yargs.option('silent', {
        default: false,
        describe: 'not console log result'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!argv.keys) {
                zignis_1.Utils.error('Please provide at least 1 key');
            }
            let keysPrefix = [];
            argv.keys.map((key) => {
                keysPrefix.push(key.split('.')[0]);
            });
            keysPrefix = zignis_1.Utils._.uniq(keysPrefix);
            const cacheKey = `${env}:${zignis_1.Utils.md5(JSON.stringify(keysPrefix))}`;
            let data;
            let consul;
            if (consulCachedKV[cacheKey]) {
                data = consulCachedKV[cacheKey];
                consul = consulCachedInstance[cacheKey];
            }
            else {
                consul = new zhike_consul_1.default(keysPrefix, consulConfig[env].host, consulConfig[env].port, {}, {
                    output: false,
                    timeout: 5000
                });
                try {
                    data = yield consul.pull(env);
                }
                catch (error) {
                    zignis_1.Utils.error(`Consul pull ${consulConfig[env].host}:${consulConfig[env].port} failed: ${error}!`);
                }
                consulCachedKV[cacheKey] = data;
                consulCachedInstance[cacheKey] = consul;
            }
            const pickNeededFromPull = {};
            argv.keys.map((key) => {
                zignis_1.Utils._.set(pickNeededFromPull, key, zignis_1.Utils._.get(data.CFG, key));
            });
            if (!argv.silent) {
                if (argv.quiet) {
                    console.log(JSON.stringify(pickNeededFromPull, null, 2));
                }
                else {
                    zignis_1.Utils.log(pickNeededFromPull);
                }
            }
            else {
                return { result: pickNeededFromPull, zhikeConsul: consul, env };
            }
        }
        catch (e) {
            zignis_1.Utils.error(e.stack);
        }
        return {};
    });
};
