"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zignis_1 = require("zignis");
exports.command = 'cli <dbKey>';
exports.desc = 'pgcli/mycli connector';
exports.builder = function (yargs) { };
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const { consul } = yield zignis_1.Utils.invokeHook('components');
        const config = yield consul.get(argv.dbKey);
        let dbConfig = zignis_1.Utils._.get(config, argv.dbKey);
        if (!dbConfig) {
            zignis_1.Utils.error('Invalid db key!');
        }
        const cmd = dbConfig.dialect === 'mysql' || dbConfig.port === 3306 ? 'mycli' : 'pgcli';
        if (!zignis_1.Utils.shell.which(cmd)) {
            zignis_1.Utils.error(`Sorry, ${cmd} needs to be installed first!`);
        }
        if (dbConfig.options) {
            dbConfig = Object.assign({}, dbConfig, dbConfig.options);
        }
        if (dbConfig.user && !dbConfig.username) {
            dbConfig.username = dbConfig.user;
        }
        if (!dbConfig.username || !dbConfig.host || !dbConfig.port || !dbConfig.database) {
            zignis_1.Utils.error('Db config in Consul is missing some key info! (host, port, database, username are required)');
        }
        const dialect = dbConfig.dialect || (cmd === 'mycli' ? 'mysql' : 'postgres');
        const connectionString = `${dialect}://${dbConfig.username}:${dbConfig.password}@${dbConfig.host}:${dbConfig.port}/${dbConfig.database}`;
        zignis_1.Utils.exec(`${cmd} ${connectionString}`);
    });
};
