"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const zignis_1 = require("zignis");
const migration = __importStar(require("../../../common/migration"));
exports.command = 'generate <dbKey> <tableName> [fieldName]';
exports.desc = 'db migration generator';
exports.aliases = ['g', 'create'];
exports.builder = function (yargs) {
    yargs.option('attributes', { default: false, describe: 'define attributes for table/field', alias: 'attrs' });
    yargs.option('rename', { describe: 'rename table/field name' });
    yargs.option('modify', { describe: 'modify field defination' });
    yargs.option('disable-timestamps', { describe: 'do not add createdAt and updatedAt fields when creating table' });
    yargs.option('only-up', { describe: 'empty down process' });
    yargs.option('simulate', { describe: 'only output in stdout', alias: 'dry-run' });
    yargs.option('reverse', { describe: 'reverse up and down' });
    yargs.option('migration-dir', { default: false, describe: 'change migration dir' });
    yargs.option('file-suffix', {
        default: false,
        describe: 'migration file suffix name, override the auto generated name'
    });
    yargs.option('index', { describe: 'add index' });
    yargs.option('typescript', { describe: 'typescript format migration file', alias: 'ts' });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { db, config } = yield zignis_1.Utils.invokeHook('components');
            let dbInstance;
            try {
                dbInstance = yield db.load(argv.dbKey);
            }
            catch (e) {
                zignis_1.Utils.error(e.message);
            }
            const dbConfig = zignis_1.Utils._.get(yield config.get(argv.dbKey), argv.dbKey);
            let tableName = dbConfig.prefix && argv.tableName.indexOf(dbConfig.prefix) !== 0
                ? dbConfig.prefix + argv.tableName
                : argv.tableName;
            let ret;
            if (argv.fieldName) {
                ret = yield migration.genMigrationForField(tableName, argv.fieldName, dbInstance, dbConfig, argv);
            }
            else {
                ret = yield migration.genMigrationForTable(tableName, dbInstance, dbConfig, argv);
            }
            const migrationDir = argv.migrationMakeDir || argv.migrationDir;
            if (argv.simulate) {
                console.log(ret);
            }
            else {
                if (!migrationDir || !fs_1.default.existsSync(migrationDir)) {
                    zignis_1.Utils.error('"migrationDir" missing in config file or not exist in current directory!');
                }
                const fileName = migration.genFileSuffix(argv);
                const filePrefix = zignis_1.Utils.day().format('YYYYMMDDHHmmssSSS');
                const migrationFile = path_1.default.resolve(migrationDir, `${filePrefix}_${zignis_1.Utils._.kebabCase(fileName)}.${argv.typescript ? 'ts' : 'js'}`);
                if (fs_1.default.existsSync(migrationFile)) {
                    zignis_1.Utils.error('File exist!');
                }
                if (!fs_1.default.existsSync(migrationFile)) {
                    fs_1.default.writeFileSync(migrationFile, ret);
                    console.log(zignis_1.Utils.chalk.green(`${migrationFile} created!`));
                }
            }
            process.exit(0);
        }
        catch (e) {
            zignis_1.Utils.error(e.stack);
        }
    });
};
