"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zignis_1 = require("zignis");
const node_sqlparser_1 = require("node-sqlparser");
const MAX_FIELDS_RENDER_TABLE = 6;
const MAX_SELECT_ROWS = 1000;
exports.command = 'query <dbKey> <sql>';
exports.desc = 'execute SQL';
exports.aliases = ['q'];
exports.builder = function (yargs) {
    yargs.option('fields', { default: false, describe: 'pick fields from query results' });
    yargs.option('pipe', { default: false, describe: 'output result using TSV format' });
    yargs.option('raw', { default: false, describe: 'output raw result' });
    yargs.option('header', { default: true, describe: 'show fields header or not' });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { db } = yield zignis_1.Utils.invokeHook('components');
            let dbInstance;
            try {
                dbInstance = yield db.load(argv.dbKey);
            }
            catch (e) {
                zignis_1.Utils.error(e.message);
            }
            let ast;
            try {
                ast = node_sqlparser_1.parse(argv.sql);
                if (ast.type && ast.type !== 'select') {
                    zignis_1.Utils.error('Only support select statement query!');
                }
            }
            catch (e) {
                zignis_1.Utils.error(e.message);
            }
            if (ast.limit) {
                if (ast.limit[1].value > MAX_SELECT_ROWS) {
                    zignis_1.Utils.error('Your query limitation must be less than 1000!');
                }
                if (ast.limit[0].value !== 0) {
                    zignis_1.Utils.error('Limitation offset not supported!');
                }
            }
            else {
                ast.limit = [{ type: 'number', value: 0 }, { type: 'number', value: '10' }];
            }
            const sql = node_sqlparser_1.stringify(ast, { offset: false });
            const results = yield dbInstance.query(sql, {
                type: dbInstance.QueryTypes.SELECT
            });
            if (results.length > 0) {
                const fields = argv.fields ? argv.fields.replace(/,/g, ' ').split(/\s+/) : Object.keys(results[0]);
                const filteredResults = results.map(function (row) {
                    let newRow = {};
                    Object.keys(row).forEach(function (field) {
                        if (fields.indexOf(field) > -1) {
                            newRow[field] = row[field];
                        }
                    });
                    return newRow;
                });
                if (argv.raw) {
                    zignis_1.Utils.log(filteredResults);
                }
                else if (argv.pipe) {
                    filteredResults.forEach(function (row) {
                        console.log(fields
                            .map(function (field) {
                            return row[field];
                        })
                            .join('\t'));
                    });
                }
                else {
                    if (fields.length > MAX_FIELDS_RENDER_TABLE) {
                        zignis_1.Utils.log(filteredResults);
                    }
                    else {
                        let rows = argv.header ? [fields] : [];
                        filteredResults.forEach(function (row) {
                            rows.push(fields.map(function (field) {
                                return row[field];
                            }));
                        });
                        console.log(zignis_1.Utils.table(rows));
                    }
                }
            }
            else {
                argv.pipe || zignis_1.Utils.warn('Query result is empty!');
            }
            process.exit(0);
        }
        catch (e) {
            zignis_1.Utils.error(e.stack);
        }
    });
};
