"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zignis_1 = require("zignis");
const kubectl_1 = require("../../../common/kubectl");
const fs_1 = __importDefault(require("fs"));
const filterFuzzy = (list, keyword) => list.filter((item) => new RegExp(keyword
    .split('')
    .map((c) => c.replace(/[.?*+^$[\]\\(){}|]/g, '\\$&'))
    .join('.*'), 'i').test(item));
const filterContain = (list, keyword) => list.filter(item => item.indexOf(keyword) > -1);
exports.command = 'list [keyword]';
exports.desc = `k8s list pods`;
exports.aliases = ['pods', 'ls'];
exports.builder = function (yargs) { };
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const namespace = argv.namespace;
        const binary = argv.binary;
        const configType = (namespace.indexOf('production') > -1 || namespace.indexOf('test') > -1) ? 'production' : 'development';
        const configPathEnv = configType === 'development' ? 'ZIGNIS_ZHIKE_K8S_DEV' : 'ZIGNIS_ZHIKE_K8S_PROD';
        const configPathKey = configType === 'development' ? 'devConfigPath' : 'prodConfigPath';
        const kubeconfigPath = process.env[configPathEnv] ? process.env[configPathEnv] : argv[configPathKey];
        if (!kubeconfigPath || !fs_1.default.existsSync(kubeconfigPath)) {
            console.error('kubeconfig file not found!');
            return;
        }
        const kubectl = new kubectl_1.Kubectl('pods', {
            binary,
            kubeconfig: kubeconfigPath,
            version: '/api/v1',
            namespace
        });
        try {
            const data = yield kubectl.list();
            const pods = [];
            data.items.forEach((item) => {
                pods.push(item.metadata.name);
            });
            let keyword, filteredPods;
            if (argv.keyword) {
                if (argv.keyword[0] === '~') {
                    keyword = argv.keyword.substring(1);
                    filteredPods = filterFuzzy(pods, keyword);
                }
                else {
                    keyword = argv.keyword;
                    filteredPods = filterContain(pods, keyword);
                }
            }
            else {
                filteredPods = pods;
            }
            filteredPods.forEach((pod) => {
                console.log(pod);
            });
        }
        catch (e) {
            zignis_1.Utils.error(e.stack);
        }
    });
};
