"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const oss_1 = __importDefault(require("../../../common/oss"));
const zignis_1 = require("zignis");
exports.command = 'list [marker]';
exports.desc = 'list keys';
exports.aliases = 'ls';
exports.builder = function (yargs) {
    yargs.option('prefix', { default: '', describe: 'search dir' });
    yargs.option('limit', { default: 10, describe: 'limit, max is 1000, default is 10' });
    yargs.option('pages', { default: 1, describe: 'pages to fetch, default is 1' });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        argv.prefix = argv.prefix ? argv.prefix.replace(/^\/+/, '') : '';
        try {
            const client = yield oss_1.default();
            let result;
            let pages = Math.max(1, argv.pages);
            while (pages > 0) {
                pages--;
                result = yield client.list({
                    marker: result && result.nextMarker ? result.nextMarker : argv.marker ? argv.marker : '',
                    prefix: argv.prefix,
                    'max-keys': argv.limit,
                    delimiter: '/'
                });
                if (result.prefixes) {
                    result.prefixes.forEach((item) => console.log(item));
                }
                if (result.objects) {
                    result.objects.forEach((item) => console.log(item.name));
                }
            }
            process.exit(0);
        }
        catch (e) {
            return zignis_1.Utils.error(e.stack);
        }
    });
};
