"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const ali_oss_1 = __importDefault(require("ali-oss"));
const zignis_1 = require("zignis");
exports.command = 'put <target> <files..>';
exports.desc = 'upload files';
exports.aliases = 'upload';
exports.builder = function (yargs) {
    yargs.option('simulate', {
        default: false,
        describe: 'Just simulate to show what files will be uploaded',
        alias: 'dry'
    });
    yargs.option('prefix', {
        default: false,
        describe: 'Only upload prefix matched file, and also prefix will be cut off'
    });
    yargs.option('header-content-type', { default: false, describe: 'Set header: Content-Type' });
    yargs.option('header-cache-control', { default: false, describe: 'Set header: Cache-Control' });
    yargs.option('header-content-disposition', { default: false, describe: 'Set header: Content-Disposition' });
    yargs.option('header-content-encoding', { default: false, describe: 'Set header: Content-Encoding' });
    yargs.option('header-expires', { default: false, describe: 'Set header: Expires' });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (argv.prefix && argv.prefix[argv.prefix.length - 1] !== '/') {
                zignis_1.Utils.error('--prefix must be end with "/"');
            }
            if (argv.target && argv.target[argv.target.length - 1] !== '/') {
                argv.target = `${argv.target}/`;
            }
            const { consul } = yield zignis_1.Utils.invokeHook('components');
            const { oss } = yield consul.get('oss');
            const client = new ali_oss_1.default({
                accessKeyId: oss.key,
                accessKeySecret: oss.secret,
                endpoint: oss.endpoint,
                bucket: oss.bucket
            });
            let matchedFiles = [];
            argv.files &&
                argv.files.forEach((filePattern) => {
                    zignis_1.Utils.glob.sync(filePattern).forEach((filePath) => {
                        matchedFiles.push(filePath);
                    });
                });
            matchedFiles = matchedFiles.filter(filePath => {
                const stat = fs_1.default.statSync(filePath);
                if (argv.prefix && filePath.indexOf(argv.prefix) !== 0) {
                    return false;
                }
                return !stat.isDirectory();
            });
            if (matchedFiles.length === 0) {
                zignis_1.Utils.error('Nothing to upload!');
            }
            if (argv.simulate) {
                matchedFiles.forEach(filePath => zignis_1.Utils.log(filePath));
                process.exit(0);
            }
            const options = {
                headers: {},
                callback: {
                    url: '',
                    host: '',
                    body: ''
                }
            };
            if (argv.headerContentType) {
                options.mime = argv.headerContentType;
            }
            if (argv.headerCacheControl) {
                options.headers['Cache-Control'] = argv.headerCacheControl;
            }
            if (argv.headerContentDisposition) {
                options.headers['Content-Disposition'] = argv.headerContentDisposition;
            }
            if (argv.headerContentEncoding) {
                options.headers['Content-Encoding'] = argv.headerContentEncoding;
            }
            if (argv.headerExpires) {
                options.headers['Expires'] = argv.headerExpires;
            }
            for (let filePath of matchedFiles) {
                let name = filePath;
                if (argv.prefix) {
                    name = filePath.substring(argv.prefix.length);
                }
                name = `${argv.target}${name}`;
                const result = yield client.put(name, filePath, options);
                if (result && result.res) {
                    if (result.res.status === 200) {
                        zignis_1.Utils.log(`${zignis_1.Utils.chalk.cyan(filePath)} uploaded to ${zignis_1.Utils.chalk.cyan(name)} ${zignis_1.Utils.chalk.green('successfully!')}`);
                    }
                    else {
                        zignis_1.Utils.log(`${zignis_1.Utils.chalk.cyan(filePath)} uploaded to ${zignis_1.Utils.chalk.cyan(name)} ${zignis_1.Utils.chalk.red('failed!')}`);
                    }
                }
            }
            process.exit(0);
        }
        catch (e) {
            return zignis_1.Utils.error(e.stack);
        }
    });
};
