"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const zhike_consul_1 = __importDefault(require("zhike-consul"));
const ioredis_1 = __importDefault(require("ioredis"));
const zignis_1 = require("zignis");
const consulConfig = require('../../../consul.json');
exports.command = 'redis cmd [arguments..]';
exports.desc = 'zhike redis tools';
exports.aliases = 'cache';
exports.builder = function (yargs) {
    yargs.option('json', {
        default: false,
        describe: 'json parse before output'
    });
    yargs.option('silent', {
        default: false,
        describe: 'do not console log the result'
    });
};
exports.handler = function (argv) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const env = zignis_1.Utils.getNodeEnv(); // development/production/test
            const consul = new zhike_consul_1.default(['redis'], consulConfig[env].host, consulConfig[env].port, {}, {
                output: false,
                timeout: 5000
            });
            const data = yield consul.pull(env);
            const config = data.CFG;
            const cache = new ioredis_1.default(config.redis);
            const ret = yield cache[argv.cmd].apply(cache, argv.arguments);
            if (!argv.silent) {
                if (argv.json) {
                    zignis_1.Utils.log(JSON.parse(ret));
                }
                else {
                    zignis_1.Utils.log(ret);
                }
                process.exit(0);
            }
            return ret;
        }
        catch (e) {
            zignis_1.Utils.error(e.stack);
        }
    });
};
