"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const sequelize_1 = require("sequelize");
const zignis_1 = require("zignis");
const fs_1 = __importDefault(require("fs"));
const consulCommand = __importStar(require("../commands/zhike/consul"));
class DatabaseLoader {
    constructor(options) {
        this.options = Object.assign({}, {
            loadReturnInstance: false,
            loadReturnModels: false,
            readonly: false
        }, options);
        this.instances = {};
    }
    get Sequelize() {
        return sequelize_1.Sequelize;
    }
    get Op() {
        return sequelize_1.Op;
    }
    /**
     * 获取数据库配置，可以直接被 Sequelize CLI 解析
     * @param {string|array} consulKey
     */
    config(consulKey) {
        return __awaiter(this, void 0, void 0, function* () {
            let dbConfig;
            if (zignis_1.Utils._.isObject(consulKey)) {
                dbConfig = consulKey;
            }
            else if (zignis_1.Utils._.isString(consulKey)) {
                const { result } = yield consulCommand.handler({ keys: [consulKey], silent: true });
                dbConfig = zignis_1.Utils._.get(result, consulKey);
            }
            if (!dbConfig) {
                throw new Error('consulKey not exist');
            }
            if (dbConfig.options) {
                dbConfig = Object.assign({}, dbConfig, dbConfig.options);
            }
            if (dbConfig.user && !dbConfig.username) {
                dbConfig.username = dbConfig.user;
            }
            return dbConfig;
        });
    }
    /**
     * 实例化数据库连接，数据库配置可以从 consul 取，也可以直接传给 load 方法
     * @param {string|array} consulKey
     * @param {string} instanceKey
     * @param {function} callback
     */
    load(consulKey, instanceKey = '', callback, opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let that = this;
            opts = Object.assign({}, {
                raw: false,
                extendModelClass: false // 后面逐步迁移到使用 init 的方式初始化模型的机制
            }, opts);
            try {
                if (zignis_1.Utils._.isFunction(instanceKey) || zignis_1.Utils._.isArray(instanceKey)) {
                    callback = instanceKey;
                    opts = callback;
                    instanceKey = consulKey;
                }
                else if (zignis_1.Utils._.isObject(instanceKey)) {
                    callback = null;
                    opts = instanceKey;
                    instanceKey = consulKey;
                }
                else if (zignis_1.Utils._.isString(instanceKey)) {
                    if (zignis_1.Utils._.isObject(callback) && !zignis_1.Utils._.isFunction(callback)) {
                        opts = callback;
                        callback = instanceKey;
                        instanceKey = '';
                    }
                    instanceKey = instanceKey || (zignis_1.Utils._.isString(consulKey) ? consulKey : zignis_1.Utils.md5(JSON.stringify(consulKey)));
                }
                else {
                    throw new Error('Undefined argument type!');
                }
                if (zignis_1.Utils._.isObject(callback) && !zignis_1.Utils._.isFunction(callback)) {
                    opts = callback;
                    callback = null;
                }
                // init db only once
                if (that.instances[instanceKey]) {
                    if (that.options.loadReturnInstance) {
                        return that.instances[instanceKey];
                    }
                    if (that.options.loadReturnModels && that.instances[instanceKey].models) {
                        return that.instances[instanceKey].models;
                    }
                    return that;
                }
                let dbConfig;
                if (zignis_1.Utils._.isObject(consulKey)) {
                    if (!instanceKey) {
                        throw new Error('The second parameter:instanceKey is required!');
                    }
                    dbConfig = consulKey;
                }
                else {
                    const { result } = yield consulCommand.handler({ keys: [consulKey], silent: true });
                    dbConfig = zignis_1.Utils._.get(result, consulKey);
                }
                if (!dbConfig) {
                    throw new Error('consulKey not exist');
                }
                if (dbConfig.options) {
                    dbConfig = Object.assign({}, dbConfig, dbConfig.options);
                }
                if (dbConfig.user && !dbConfig.username) {
                    dbConfig.username = dbConfig.user;
                }
                let sequelize = new sequelize_1.Sequelize(dbConfig.database, dbConfig.username, dbConfig.password, {
                    dialect: dbConfig.dialect,
                    host: dbConfig.host,
                    port: dbConfig.port,
                    timezone: '+08:00',
                    logging: undefined,
                    pool: {
                        max: zignis_1.Utils._.isObject(dbConfig.pool) ? (dbConfig.pool.max || 50) : (dbConfig.pool || 50)
                    },
                    query: {
                        raw: opts.raw || false
                    }
                });
                function forbiddenMethod() {
                    throw new Error('Dangerous method forbidden!');
                }
                // 防止误操作，删除、清空整个库
                sequelize.drop = forbiddenMethod; // 删除所有的表
                sequelize.truncate = forbiddenMethod; // 清空所有的表
                sequelize.dropAllSchemas = forbiddenMethod; // 删除所有的 postgres schema，即删掉整个数据库
                sequelize.dropSchema = forbiddenMethod; // 删除一个 postgres schema，一般也相当于删掉整个数据库
                yield sequelize.authenticate();
                that.instances[instanceKey] = sequelize;
                const queryInterface = sequelize.getQueryInterface();
                const tables = yield queryInterface.showAllTables();
                const tableInfos = yield Promise.all(tables.map((table) => {
                    return queryInterface.describeTable(table).catch(() => false);
                }));
                const combinedTableInfos = zignis_1.Utils._.zipObject(tables, tableInfos);
                Object.keys(combinedTableInfos).forEach(table => {
                    const tableInfo = combinedTableInfos[table];
                    if (!tableInfo)
                        return;
                    const newTableInfo = {};
                    const newTableFields = [];
                    let tableAutoIncrementFieldExisted = false;
                    Object.keys(tableInfo).map(field => {
                        const newField = field.replace(/(_.)/g, function (word) {
                            return word[1].toUpperCase();
                        });
                        tableInfo[field].field = field;
                        // Only one autoincrement field allowed, we should put autoIncrement at the first of the table
                        if (tableAutoIncrementFieldExisted && tableInfo[field].autoIncrement) {
                            delete tableInfo[field].autoIncrement;
                        }
                        // for PG, check autoIncrement rule
                        if (/^nextval\(.*?::regclass\)$/.test(tableInfo[field].defaultValue)) {
                            delete tableInfo[field].defaultValue;
                            tableInfo[field].autoIncrement = true;
                            tableAutoIncrementFieldExisted = true;
                        }
                        newTableInfo[newField] = tableInfo[field];
                        newTableFields.push(newField);
                    });
                    if (!tableAutoIncrementFieldExisted && newTableInfo.id) {
                        newTableInfo.id.allowNull = true;
                    }
                    const modelName = table.indexOf(dbConfig.prefix) > -1
                        ? table.substring(dbConfig.prefix.length).replace(/(_.)/g, function (word) {
                            return word[1].toUpperCase();
                        })
                        : table.replace(/(_.)/g, function (word) {
                            return word[1].toUpperCase();
                        });
                    const modelNameUpper = modelName.replace(/( |^)[a-z]/g, L => L.toUpperCase());
                    try {
                        let options = {
                            tableName: table,
                            modelName: modelNameUpper,
                            sequelize
                        };
                        if (newTableFields.indexOf('createdAt') === -1) {
                            options.createdAt = false;
                        }
                        else {
                            options.createdAt = newTableInfo['createdAt'].field;
                        }
                        if (newTableFields.indexOf('updatedAt') === -1) {
                            options.updatedAt = false;
                        }
                        else {
                            options.updatedAt = newTableInfo['updatedAt'].field;
                        }
                        let model;
                        if (!opts.extendModelClass) {
                            model = sequelize.define(modelNameUpper, newTableInfo, options);
                        }
                        else {
                            let modelFilePath;
                            try {
                                modelFilePath = require.resolve(`${callback}/${modelNameUpper}`);
                            }
                            catch (e) {
                                if (e.code !== 'MODULE_NOT_FOUND') {
                                    console.error(e.message);
                                }
                            }
                            if (zignis_1.Utils._.isString(callback) && modelFilePath && fs_1.default.existsSync(modelFilePath)) {
                                model = (require(modelFilePath)).init(newTableInfo, options);
                            }
                            else {
                                model = sequelize.define(modelNameUpper, newTableInfo, options);
                            }
                        }
                        model.drop = forbiddenMethod; // 以防误删表
                        model.sync = forbiddenMethod;
                        if (that.options.readonly && zignis_1.Utils.getNodeEnv() === 'production') {
                            model.upsert = forbiddenMethod;
                            model.truncate = forbiddenMethod;
                            model.destroy = forbiddenMethod;
                            model.restore = forbiddenMethod;
                            model.update = forbiddenMethod;
                            model.create = forbiddenMethod;
                            model.findOrCreate = forbiddenMethod;
                            model.bulkCreate = forbiddenMethod;
                            model.removeAttribute = forbiddenMethod;
                        }
                    }
                    catch (e) { }
                });
                if (!opts.extendModelClass) {
                    // load 函数的callback参数可以是
                    // 1, 一个回调函数的数组，
                    // 2, 一组路径
                    // 3，一个回调函数
                    // 4，一个路径
                    if (callback) {
                        if (zignis_1.Utils._.isArray(callback)) {
                            callback.map((cb) => {
                                if (zignis_1.Utils._.isFunction(cb)) {
                                    cb(sequelize.models, sequelize);
                                }
                                else if (zignis_1.Utils._.isString(cb)) {
                                    // implicitly means to call this.associate, and cb is actually modealPath
                                    that.associate(cb)(sequelize.models, sequelize);
                                }
                            });
                        }
                        else {
                            if (zignis_1.Utils._.isFunction(callback)) {
                                callback(sequelize.models, sequelize);
                            }
                            else if (zignis_1.Utils._.isString(callback)) {
                                // implicitly means to call this.associate, and callback is actually modealPath
                                that.associate(callback)(sequelize.models, sequelize);
                            }
                        }
                    }
                }
                else {
                    Object.keys(sequelize.models).forEach(function (modelName) {
                        let model = sequelize.models[modelName];
                        if (zignis_1.Utils._.isFunction(model.associate)) {
                            model.associate(sequelize.models);
                        }
                    });
                }
                if (that.options.loadReturnInstance) {
                    return that.instances[instanceKey];
                }
                if (that.options.loadReturnModels && that.instances[instanceKey].models) {
                    return that.instances[instanceKey].models;
                }
                return that;
            }
            catch (e) {
                throw new Error(e.stack);
            }
        });
    }
    /**
     * 处理模型的关联关系
     * @param {string} modelPath
     */
    // @ts-ignore
    associate(modelPath) {
        return function (models, sequelize) {
            Object.keys(models).forEach(modelName => {
                if (fs_1.default.existsSync(`${modelPath}/${modelName}.js`)) {
                    let model = sequelize.models[modelName];
                    let modelExtend = require(`${modelPath}/${modelName}`);
                    if (zignis_1.Utils._.isFunction(modelExtend)) {
                        const ret = modelExtend.bind(model)(sequelize.models, sequelize);
                        if (ret) {
                            sequelize.models[modelName] = ret;
                        }
                    }
                    else {
                        throw new Error('Model extension must be a function.');
                    }
                }
            });
        };
    }
}
module.exports = DatabaseLoader;
