"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const zignis_1 = require("zignis");
const ioredis_1 = __importDefault(require("ioredis"));
const mq_1 = __importDefault(require("./mq"));
exports.mq = mq_1.default;
const oss_1 = __importDefault(require("./oss"));
exports.oss = oss_1.default;
const elasticsearch_1 = __importDefault(require("./elasticsearch"));
exports.elasticsearch = elasticsearch_1.default;
const consulCommand = __importStar(require("../commands/zhike/consul"));
const db_1 = __importDefault(require("./db"));
const api_1 = __importDefault(require("./api"));
exports.api = api_1.default;
const dbForComponent = new db_1.default({ loadReturnInstance: true });
exports.dbForComponent = dbForComponent;
const dbForRepl = new db_1.default({ readonly: true, loadReturnModels: true });
exports.dbForRepl = dbForRepl;
const debug = zignis_1.Utils.debug('zignis-plugin-zhike:utils');
const config = {
    get(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                keys = Array.isArray(keys) ? keys : zignis_1.Utils.splitComma(keys);
                const { result } = yield consulCommand.handler({ keys, silent: true });
                return result;
            }
            catch (e) {
                throw new Error(e.stack);
            }
        });
    },
    getAndWatch(keys, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const keysArray = Array.isArray(keys) ? keys : zignis_1.Utils.splitComma(keys);
                const { result, zhikeConsul, env } = yield consulCommand.handler({ keys: keysArray, silent: true });
                const consul = zhikeConsul.consul;
                const watch = function (key) {
                    let watch = consul.watch({ method: consul.kv.get, options: { key } });
                    watch.once('change', function () {
                        // 初始会有一次 change 事件，忽略第一次的事件
                        watch.on('change', function (data) {
                            let value = JSON.parse(data.Value)[env];
                            debug(`CFG ${key} changed to:`, value);
                            callback && callback(key, value);
                        });
                        watch.on('error', function (err) {
                            // always received errors, but watch still working, use this event listenner to skip error
                            // console.log('error:', err);
                        });
                    });
                };
                keysArray.map(watch);
                return result;
            }
            catch (e) {
                throw new Error(e.stack);
            }
        });
    }
};
exports.config = config;
const redisInstance = () => __awaiter(void 0, void 0, void 0, function* () {
    const { redis: redisConfig } = yield config.get('redis');
    const redis = new ioredis_1.default(redisConfig);
    return redis;
});
exports.redisInstance = redisInstance;
