"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const zignis_1 = require("zignis");
exports.command = 'cron <name>';
exports.desc = 'Generate a zignis cron job file';
exports.builder = function (yargs) {
    yargs.option('typescript', {
        alias: 'ts',
        describe: 'generate typescript style code'
    });
};
exports.handler = function (argv) {
    const cronDir = argv.cronMakeDir || argv.cronDir;
    if (!cronDir || !fs_1.default.existsSync(cronDir)) {
        console.log(zignis_1.Utils.chalk.red('"cronDir" missing in config file or not exist in current directory!'));
        return;
    }
    const filePrefix = zignis_1.Utils.day().format('YYYYMMDDHHmmssSSS');
    const cronFile = path_1.default.resolve(cronDir, `${filePrefix}_${zignis_1.Utils._.kebabCase(argv.name)}.${argv.typescript ? 'ts' : 'js'}`);
    if (fs_1.default.existsSync(cronFile)) {
        console.log(zignis_1.Utils.chalk.red('Script file exist!'));
        return;
    }
    let code;
    if (argv.typescript) {
        code = `const sleep = ms => new Promise(resolve => setTimeout(resolve, ms))

// 示例 Job Actions
const demoAction = async function demo() {
  console.log('Demo job action')
  await sleep(60000)
}

export const schedule = '* * * * *'
export const duration = 60000
export const actions = [demoAction]
export const disabled = false
export const env = 'production'
`;
    }
    else {
        code = `const sleep = ms => new Promise(resolve => setTimeout(resolve, ms))

// 示例 Job Actions
const demoAction = async function demo() {
  console.log('Demo job action')
  await sleep(60000)
}

exports.schedule = '* * * * *'
exports.duration = 60000
exports.actions = [demoAction]
exports.disabled = false
`;
    }
    if (!fs_1.default.existsSync(cronFile)) {
        fs_1.default.writeFileSync(cronFile, code);
        console.log(zignis_1.Utils.chalk.green(`${cronFile} created!`));
    }
};
