export declare const hook_hook: {
    zhike_cron: string;
    zhike_component: string;
    zhike_repl: string;
};
export declare const hook_new_repo: {
    zignis_taro_starter: {
        repo: string;
        branch: string;
        alias: string[];
    };
    weapp_starter: {
        repo: string;
        branch: string;
        alias: string[];
    };
    zignis_backend_starter: {
        repo: string;
        branch: string;
        alias: string[];
    };
    backend_scaffold: {
        repo: string;
        branch: string;
        alias: string[];
    };
};
/**
 * Implement hook: repl.
 * Add Zhike resources into Zignis REPL mode.
 * @example
 * /// Fetch zhike consul config，alias: zhike.config
 * $ zignis repl
 * >>> await zhike.consul.get('db.user', 'oss')
 * @example
 * /// Support all ioredis apis, alias: zhike.cache
 * $ zignis repl
 * >>> await zhike.redis.keys('*')
 * @example
 * /// All database instances are loaded into zhike.db.instances
 * /// Database instances are Sequelize model instances
 * /// You can not do dangerous Sequelize operations in REPL mode
 * $ zignis repl
 * >>> await zhike.db.load('db.user', 'user')
 * >>> const { Account } = zhike.db.instances.user.models
 * >>> await Account.count()
 * 27378
 * @returns {object} Zhike resources, for now include consul, redis, db.
 */
export declare const hook_repl: () => Promise<object>;
/**
 * Implement hook: components.
 * Add Zhike resources into plugins which depend on this plugin, or Zignis scripts.
 * @example
 * /// Used in command defination
 * const { Utils } = require('zignis')
 * exports.handler = async function(argv) {
 *   const { db } = await Utils.invokeHook('components')
 *   /// Here, Sequelize instance is returned directly, different from repl hook
 *   const userDb = await db.load('db.user', 'user', db.associate('./models'))
 *   const { Account } = userDb.models
 *   const count = await Account.count()
 * }
 * @example
 * /// Zignis script db access demo
 * module.exports = async function(components) {
 *   const { config } = await Utils.invokeHook('components')
 *   console.log(await config.get('oss'))
 *   console.log('Start to draw your dream code!')
 *   process.exit(0)
 * }
 * @example
 * /// Consul config watch demo
 * const { consul } = await Utils.invokeHook('components')
 * const config = await consul.getAndWatch('socialPrivate,oss', function(key, value) {
 *   config[key] = value
 *   console.log(`Consul key: ${key} changed to:`, value)
 * })
 * @returns {object} Zhike resources, for now include consul, redis, db.
 */
export declare const hook_components: () => Promise<object>;
export declare const hook_beforeCommand: () => Promise<{
    zignisPluginTen: (argv: any, yargs: any) => void;
}>;
